@@ -235,7 +235,7 @@ REST_API_VERSION_HISTORY = """REST API Version History:
              ``POST /flavors/{flavor_id}/os-extra_specs`` and
              ``PUT /flavors/{flavor_id}/os-extra_specs/{id}`` APIs.
     * 2.87 - Adds support for rescuing boot from volume instances when the
-             compute host reports the COMPUTE_BFV_RESCUE capability trait.
+             compute host reports the COMPUTE_RESCUE_BFV capability trait.
     * 2.88 - Drop statistics-style fields from the ``/os-hypervisors/detail``
              and ``/os-hypervisors/{hypervisor_id}`` APIs, and remove the
              ``/os-hypervisors/statistics`` and
@@ -253,6 +253,10 @@ REST_API_VERSION_HISTORY = """REST API Version History:
              ``POST /os-keypairs`` and allow including @ and dot (.) characters
              in keypair name.
     * 2.93 - Add support for volume backed server rebuild.
+    * 2.94 - Allow FQDN in server hostname.
+    * 2.95 - Evacuate will now stop instance at destination.
+    * 2.96 - Add support for returning pinned_availability_zone in
+             ``server show`` and ``server list --long`` responses.
 """
 
 # The minimum and maximum versions of the API supported
@@ -261,7 +265,7 @@ REST_API_VERSION_HISTORY = """REST API Version History:
 # Note(cyeoh): This only applies for the v2.1 API once microversions
 # support is fully merged. It does not affect the V2 API.
 _MIN_API_VERSION = '2.1'
-_MAX_API_VERSION = '2.93'
+_MAX_API_VERSION = '2.96'
 DEFAULT_API_VERSION = _MIN_API_VERSION
 
 # Almost all proxy APIs which are related to network, images and baremetal
@@ -385,10 +389,10 @@ class APIVersionRequest(object):
         If self is null then raise ValueError
         """
 
-        if self.is_null():
-            raise ValueError
         if max_version.is_null() and min_version.is_null():
             return True
+        elif self.is_null():
+            raise ValueError
         elif max_version.is_null():
             return min_version <= self
         elif min_version.is_null():
