@@ -21,7 +21,6 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
 from oslo_utils import strutils
-import six
 import webob
 
 from nova.api.openstack import api_version_request as api_version
@@ -30,6 +29,7 @@ from nova.api import wsgi
 from nova import exception
 from nova import i18n
 from nova.i18n import _
+from nova import version
 
 
 LOG = logging.getLogger(__name__)
@@ -142,7 +142,7 @@ class Request(wsgi.Request):
         # no match. This is also little tricky that 'default' value cannot be
         # None. At least one of default_tag or default must be supplied as
         # an argument to the method, to define the defaulting behavior.
-        # So passing a sentinal value to return None from this function.
+        # So passing a sentinel value to return None from this function.
         best_match = self.accept_language.lookup(
             i18n.get_available_languages(), default='fake_LANG')
 
@@ -220,7 +220,7 @@ class JSONDictSerializer(ActionDispatcher):
         return self.dispatch(data, action=action)
 
     def default(self, data):
-        return six.text_type(jsonutils.dumps(data))
+        return str(jsonutils.dumps(data))
 
 
 def response(code):
@@ -293,8 +293,8 @@ class ResponseObject(object):
             response.headers[hdr] = encodeutils.safe_decode(
                     encodeutils.safe_encode(val))
         # Deal with content_type
-        if not isinstance(content_type, six.text_type):
-            content_type = six.text_type(content_type)
+        if not isinstance(content_type, str):
+            content_type = str(content_type)
         # In Py3.X Headers must be a str.
         response.headers['Content-Type'] = encodeutils.safe_decode(
                 encodeutils.safe_encode(content_type))
@@ -396,8 +396,8 @@ class Resource(wsgi.Application):
         """:param controller: object that implement methods created by routes
                               lib
         """
-
         self.controller = controller
+        self.sub_controllers = []
 
         self.default_serializers = dict(json=JSONDictSerializer)
 
@@ -413,6 +413,13 @@ class Resource(wsgi.Application):
         for key, method_name in actions.items():
             self.wsgi_actions[key] = getattr(controller, method_name)
 
+    def register_subcontroller_actions(self, sub_controller):
+        """Registers sub-controller actions with this resource."""
+        self.sub_controllers.append(sub_controller)
+        actions = getattr(sub_controller, 'wsgi_actions', {})
+        for key, method_name in actions.items():
+            self.wsgi_actions[key] = getattr(sub_controller, method_name)
+
     def get_action_args(self, request_environment):
         """Parse dictionary created by routes library."""
 
@@ -506,7 +513,7 @@ class Resource(wsgi.Application):
         if body:
             msg = _("Action: '%(action)s', calling method: %(meth)s, body: "
                     "%(body)s") % {'action': action,
-                                   'body': six.text_type(body, 'utf-8'),
+                                   'body': str(body, 'utf-8'),
                                    'meth': str(meth)}
             LOG.debug(strutils.mask_password(msg))
         else:
@@ -562,8 +569,8 @@ class Resource(wsgi.Application):
 
         if hasattr(response, 'headers'):
             for hdr, val in list(response.headers.items()):
-                if not isinstance(val, six.text_type):
-                    val = six.text_type(val)
+                if not isinstance(val, str):
+                    val = str(val)
                 # In Py3.X Headers must be a string
                 response.headers[hdr] = encodeutils.safe_decode(
                         encodeutils.safe_encode(val))
@@ -644,6 +651,19 @@ def action(name):
     return decorator
 
 
+def removed(version: str, reason: str):
+    """Mark a function as removed.
+
+    The given reason will be stored as an attribute of the function.
+    """
+    def decorator(func):
+        func.removed = True
+        func.removed_version = reason
+        func.removed_reason = reason
+        return func
+    return decorator
+
+
 def expected_errors(errors):
     """Decorator for v2.1 API methods which specifies expected exceptions.
 
@@ -671,6 +691,15 @@ def expected_errors(errors):
                     # calls. ResourceExceptionHandler silently
                     # converts NotAuthorized to HTTPForbidden
                     raise
+                elif isinstance(exc, exception.NotSupported):
+                    # Note(gmann): Special case to handle
+                    # NotSupported exceptions. We want to raise 400 BadRequest
+                    # for the NotSupported exception which is basically used
+                    # to raise for not supported features. Converting it here
+                    # will avoid converting every NotSupported inherited
+                    # exception in API controller.
+                    raise webob.exc.HTTPBadRequest(
+                            explanation=exc.format_message())
                 elif isinstance(exc, exception.ValidationError):
                     # Note(oomichi): Handle a validation error, which
                     # happens due to invalid API parameters, as an
@@ -684,9 +713,10 @@ def expected_errors(errors):
                     raise
 
                 LOG.exception("Unexpected exception in API method")
-                msg = _('Unexpected API Error. Please report this at '
-                    'http://bugs.launchpad.net/nova/ and attach the Nova '
-                    'API log if possible.\n%s') % type(exc)
+                msg = _("Unexpected API Error. "
+                        "{support}\n{exc}").format(
+                            support=version.support_string(),
+                            exc=type(exc))
                 raise webob.exc.HTTPInternalServerError(explanation=msg)
 
         return wrapped
