@@ -192,14 +192,28 @@ class ImageMetaProps(base.NovaObject):
     #                     'hw_ephemeral_encryption_format' fields
     # Version 1.33: Added 'hw_locked_memory' field
     # Version 1.34: Added 'hw_viommu_model' field
+    # Version 1.35: Added 'hw_virtio_packed_ring' field
+    # Version 1.36: Added 'hw_maxphysaddr_mode' and
+    #                     'hw_maxphysaddr_bits' field
+    # Version 1.37: Added 'hw_ephemeral_encryption_secret_uuid' field
+    # Version 1.38: Added 'hw_firmware_stateless' field
     # NOTE(efried): When bumping this version, the version of
     # ImageMetaPropsPayload must also be bumped. See its docstring for details.
-    VERSION = '1.34'
+    VERSION = '1.38'
 
     def obj_make_compatible(self, primitive, target_version):
         super(ImageMetaProps, self).obj_make_compatible(primitive,
                                                         target_version)
         target_version = versionutils.convert_version_to_tuple(target_version)
+        if target_version < (1, 38):
+            primitive.pop('hw_firmware_stateless', None)
+        if target_version < (1, 37):
+            primitive.pop('hw_ephemeral_encryption_secret_uuid', None)
+        if target_version < (1, 36):
+            primitive.pop('hw_maxphysaddr_mode', None)
+            primitive.pop('hw_maxphysaddr_bits', None)
+        if target_version < (1, 35):
+            primitive.pop('hw_virtio_packed_ring', None)
         if target_version < (1, 34):
             primitive.pop('hw_viommu_model', None)
         if target_version < (1, 33):
@@ -367,6 +381,9 @@ class ImageMetaProps(base.NovaObject):
         # This indicates the guest needs UEFI firmware
         'hw_firmware_type': fields.FirmwareTypeField(),
 
+        # This indicates the guest needs stateless firmware
+        'hw_firmware_stateless': fields.FlexibleBooleanField(),
+
         # name of the input bus type to use, e.g. usb, virtio
         'hw_input_bus': fields.InputBusField(),
 
@@ -472,6 +489,17 @@ class ImageMetaProps(base.NovaObject):
         # encryption format to be used when ephemeral encryption is enabled
         'hw_ephemeral_encryption_format':
             fields.BlockDeviceEncryptionFormatTypeField(),
+        # encryption secret uuid string for passphrase in the image
+        'hw_ephemeral_encryption_secret_uuid': fields.UUIDField(),
+
+        # boolean - If true, this will enable the virtio packed ring feature
+        'hw_virtio_packed_ring': fields.FlexibleBooleanField(),
+
+        # Control mode for the physical memory address bit of Libvirt guests.
+        'hw_maxphysaddr_mode': fields.MaxPhysAddrModeField(),
+
+        # Control bits for the physical memory address bit of Libvirt guests.
+        'hw_maxphysaddr_bits': fields.IntegerField(),
 
         # if true download using bittorrent
         'img_bittorrent': fields.FlexibleBooleanField(),
