@@ -18,20 +18,23 @@ Provides common functionality for integrated unit tests
 """
 
 import collections
+import datetime
 import random
+import re
 import string
 import time
 
 from oslo_concurrency import lockutils
 from oslo_log import log as logging
 import oslo_messaging as messaging
+from oslo_utils.fixture import uuidsentinel as uuids
 
 from nova.compute import instance_actions
 from nova.compute import rpcapi as compute_rpcapi
 from nova.compute import utils as compute_utils
 import nova.conf
 from nova import context
-from nova.db import api as db
+from nova.db.main import api as db
 import nova.image.glance
 from nova import objects
 from nova.objects import base as objects_base
@@ -40,10 +43,8 @@ from nova import test
 from nova.tests import fixtures as nova_fixtures
 from nova.tests.functional.api import client as api_client
 from nova.tests.functional import fixtures as func_fixtures
-from nova.tests.unit import cast_as_call
-from nova.tests.unit import fake_notifier
-from nova.tests.unit import policy_fixture
 from nova import utils
+import typing as ty
 
 
 CONF = nova.conf.CONF
@@ -86,7 +87,7 @@ class StubComputeRPCAPI(compute_rpcapi.ComputeAPI):
     @property
     def router(self):
         with lockutils.lock('compute-rpcapi-router'):
-            target = messaging.Target(topic='compute', version='5.0')
+            target = messaging.Target(topic='compute', version='6.0')
             version_cap = self.version
             serializer = objects_base.NovaObjectSerializer()
             rpc.get_client(target, version_cap, serializer)
@@ -94,8 +95,35 @@ class StubComputeRPCAPI(compute_rpcapi.ComputeAPI):
             return rpc.ClientRouter(default_client)
 
 
+# placeholder used as a default parameter value to distinguish between the case
+# when the parameter is specified by the caller with None from the case when it
+# was not specified
+NOT_SPECIFIED = object()
+
+
 class InstanceHelperMixin:
 
+    def _wait_for_service_parameter(
+        self, service_hostname, service_binary, expected_params,
+        max_retries=10, api=None
+    ):
+        api = api or getattr(self, 'admin_api', self.api)
+
+        retry_count = 0
+        while True:
+            service = api.get_services(
+                host=service_hostname,
+                binary=service_binary)[0]
+            if all([service[attr] == expected_params[attr]
+                    for attr in expected_params]):
+                break
+            retry_count += 1
+            if retry_count == max_retries:
+                self.fail(
+                    f'Wait for service parameter change failed, '
+                    f'expected_params={expected_params}, service={service}')
+            time.sleep(0.5)
+
     def _wait_for_server_parameter(
             self, server, expected_params, max_retries=10, api=None):
         api = api or getattr(self, 'admin_api', self.api)
@@ -177,6 +205,37 @@ class InstanceHelperMixin:
             'actions: %s. Events in the last matching action: %s'
             % (event_name, actions, events))
 
+    def _wait_for_volume_attach(self, server_id, volume_id):
+        timeout = 0.0
+        while timeout < 10.0:
+            try:
+                self.api.get_server_volume(server_id, volume_id)
+                return
+            except api_client.OpenStackApiNotFoundException:
+                time.sleep(.1)
+                timeout += .1
+
+        attached_vols = self.api.get_server_volumes(server_id)
+        self.fail('Timed out waiting for volume %s to be attached to '
+                      'server %s. Currently attached volumes: %s' %
+                      (volume_id, server_id, attached_vols))
+
+    def _wait_for_volume_detach(self, server_id, volume_id):
+        timeout = 0.0
+
+        while timeout < 10.0:
+            try:
+                self.api.get_server_volume(server_id, volume_id)
+                time.sleep(.1)
+                timeout += .1
+            except api_client.OpenStackApiNotFoundException:
+                return
+
+        attached_vols = self.api.get_server_volumes(server_id)
+        self.fail('Timed out waiting for volume %s to be detached from '
+                  'server %s. Currently attached volumes: %s' %
+                  (volume_id, server_id, attached_vols))
+
     def _assert_resize_migrate_action_fail(self, server, action, error_in_tb):
         """Waits for the conductor_migrate_server action event to fail for
         the given action and asserts the error is in the event traceback.
@@ -191,6 +250,27 @@ class InstanceHelperMixin:
         self.assertIn(error_in_tb, event['traceback'])
         return event
 
+    def _assert_build_request_success(self, server_request):
+        server = self.api.post_server({'server': server_request})
+        self._wait_for_state_change(server, 'ACTIVE')
+        return server['id']
+
+    def _assert_build_request_schedule_failure(self, server_request):
+        server = self.api.post_server({'server': server_request})
+        self._wait_for_state_change(server, 'ERROR')
+
+    def _assert_bad_build_request_error(self, server_request):
+        ex = self.assertRaises(
+            api_client.OpenStackApiException, self.api.post_server,
+            {'server': server_request})
+        self.assertEqual(400, ex.response.status_code)
+
+    def _assert_build_request_error(self, server_request):
+        ex = self.assertRaises(
+            api_client.OpenStackApiException, self.api.post_server,
+            {'server': server_request})
+        self.assertEqual(500, ex.response.status_code)
+
     def _wait_for_migration_status(self, server, expected_statuses):
         """Waits for a migration record with the given statuses to be found
         for the given server, else the test fails. The migration record, if
@@ -216,9 +296,9 @@ class InstanceHelperMixin:
                 server['id'], expected_statuses, actual_status,
             ))
 
-    def _wait_for_log(self, log_line):
+    def _wait_for_log(self, log_line_regex):
         for i in range(10):
-            if log_line in self.stdlog.logger.output:
+            if re.search(log_line_regex, self.stdlog.logger.output):
                 return
             time.sleep(0.5)
 
@@ -316,6 +396,32 @@ class InstanceHelperMixin:
 
         return flavor['id']
 
+    def _create_image(self, metadata):
+        image = {
+            'id': 'c456eb30-91d7-4f43-8f46-2efd9eccd744',
+            'name': 'fake-image-custom-property',
+            'created_at': datetime.datetime(2011, 1, 1, 1, 2, 3),
+            'updated_at': datetime.datetime(2011, 1, 1, 1, 2, 3),
+            'deleted_at': None,
+            'deleted': False,
+            'status': 'active',
+            'is_public': False,
+            'container_format': 'raw',
+            'disk_format': 'raw',
+            'size': '25165824',
+            'min_ram': 0,
+            'min_disk': 0,
+            'protected': False,
+            'visibility': 'public',
+            'tags': ['tag1', 'tag2'],
+            'properties': {
+                'kernel_id': 'nokernel',
+                'ramdisk_id': 'nokernel',
+            },
+        }
+        image['properties'].update(metadata)
+        return self.glance.create(None, image)
+
     def _build_server(self, name=None, image_uuid=None, flavor_id=None,
                       networks=None, az=None, host=None):
         """Build a request for the server create API.
@@ -412,15 +518,41 @@ class InstanceHelperMixin:
         self.api.post_server_action(
             server['id'], {'reboot': {'type': 'HARD' if hard else 'SOFT'}},
         )
-        fake_notifier.wait_for_versioned_notifications('instance.reboot.end')
+        self.notifier.wait_for_versioned_notifications('instance.reboot.end')
         return self._wait_for_state_change(server, expected_state)
 
+    def _show_server(self, server):
+        """This method is to retrieve fresh copy of target server object
+        """
+        try:
+            return self.api.get_server(server['id'])
+        except api_client.OpenStackApiNotFoundException:
+            return server
+
+    def _attach_interface(self, server, port_uuid):
+        """attach a neutron port to a server."""
+        body = {
+            "interfaceAttachment": {
+                "port_id": port_uuid
+            }
+        }
+        attachment = self.api.attach_interface(server['id'], body)
+        self.notifier.wait_for_versioned_notifications(
+            'instance.interface_attach.end')
+        return attachment
+
+    def _detach_interface(self, server, port_uuid):
+        """detach a neutron port form a server."""
+        self.api.detach_interface(server['id'], port_uuid)
+        self.notifier.wait_for_versioned_notifications(
+            'instance.interface_detach.end')
+
     def _rebuild_server(self, server, image_uuid, expected_state='ACTIVE'):
         """Rebuild a server."""
         self.api.post_server_action(
             server['id'], {'rebuild': {'imageRef': image_uuid}},
         )
-        fake_notifier.wait_for_versioned_notifications('instance.rebuild.end')
+        self.notifier.wait_for_versioned_notifications('instance.rebuild.end')
         return self._wait_for_state_change(server, expected_state)
 
     def _migrate_server(self, server, host=None):
@@ -432,7 +564,7 @@ class InstanceHelperMixin:
     def _resize_server(self, server, flavor_id):
         self.api.post_server_action(
             server['id'], {'resize': {'flavorRef': flavor_id}})
-        fake_notifier.wait_for_versioned_notifications('instance.resize.end')
+        self.notifier.wait_for_versioned_notifications('instance.resize.end')
         return self._wait_for_state_change(server, 'VERIFY_RESIZE')
 
     def _confirm_resize(self, server, *, cross_cell=False):
@@ -455,7 +587,7 @@ class InstanceHelperMixin:
         # dest host revert_resize method but the allocations are cleaned up
         # in the source host finish_revert_resize method so we need to wait
         # for the finish_revert_resize method to complete.
-        fake_notifier.wait_for_versioned_notifications(
+        self.notifier.wait_for_versioned_notifications(
             'instance.resize_revert.end')
         return server
 
@@ -466,21 +598,21 @@ class InstanceHelperMixin:
         self.api.post_server_action(
             server['id'],
             {'os-migrateLive': {'host': None, 'block_migration': 'auto'}})
-        self._wait_for_state_change(server, server_expected_state)
         self._wait_for_migration_status(server, [migration_expected_state])
+        return self._wait_for_state_change(server, server_expected_state)
 
     _live_migrate_server = _live_migrate
 
     def _suspend_server(self, server, expected_state='SUSPENDED'):
         """Suspend a server."""
         self.api.post_server_action(server['id'], {'suspend': {}})
-        fake_notifier.wait_for_versioned_notifications('instance.suspend.end')
+        self.notifier.wait_for_versioned_notifications('instance.suspend.end')
         return self._wait_for_state_change(server, expected_state)
 
     def _resume_server(self, server, expected_state='ACTIVE'):
         """Resume a server."""
         self.api.post_server_action(server['id'], {'resume': {}})
-        fake_notifier.wait_for_versioned_notifications('instance.resume.end')
+        self.notifier.wait_for_versioned_notifications('instance.resume.end')
         return self._wait_for_state_change(server, expected_state)
 
     def _shelve_server(self, server, expected_state='SHELVED_OFFLOADED'):
@@ -488,17 +620,127 @@ class InstanceHelperMixin:
         self.api.post_server_action(server['id'], {'shelve': {}})
         return self._wait_for_state_change(server, expected_state)
 
+    def _shelve_offload_server(self, server):
+        """Shelve offload a server."""
+        self.api.post_server_action(server['id'], {'shelveOffload': {}})
+        self._wait_for_server_parameter(server,
+                                        {'status': 'SHELVED_OFFLOADED',
+                                         'OS-EXT-SRV-ATTR:host': None,
+                                         'OS-EXT-AZ:availability_zone': ''})
+
     def _unshelve_server(self, server, expected_state='ACTIVE'):
         """Unshelve a server."""
-        self.api.post_server_action(server['id'], {'unshelve': {}})
+        self.api.post_server_action(server['id'], {'unshelve': None})
         return self._wait_for_state_change(server, expected_state)
 
-    def _evacuate_server(self, server, host, expected_state='ACTIVE'):
+    def _evacuate_server(
+            self, server, extra_post_args=None, expected_host=None,
+            expected_state='SHUTOFF', expected_task_state=NOT_SPECIFIED,
+            expected_migration_status='done'):
         """Evacuate a server."""
-        self.api.post_server_action(server['id'], {'evacuate': {}})
-        self._wait_for_server_parameter(
-            self.server, {'OS-EXT-SRV-ATTR:host': host,
-                          'status': expected_state})
+        api = getattr(self, 'admin_api', self.api)
+
+        post = {'evacuate': {}}
+        if extra_post_args:
+            post['evacuate'].update(extra_post_args)
+
+        expected_result = {'status': expected_state}
+        if expected_host:
+            expected_result['OS-EXT-SRV-ATTR:host'] = expected_host
+        if expected_task_state is not NOT_SPECIFIED:
+            expected_result['OS-EXT-STS:task_state'] = expected_task_state
+
+        api.post_server_action(server['id'], post)
+
+        # NOTE(gibi): The order of waiting for the migration and returning
+        # a fresh server from _wait_for_server_parameter is important as
+        # the compute manager sets status of the instance before sets the
+        # host and finally sets the migration status. So waiting for the
+        # migration first makes the returned server object more consistent.
+        self._wait_for_migration_status(server, [expected_migration_status])
+        return self._wait_for_server_parameter(server, expected_result)
+
+    def _start_server(self, server):
+        self.api.post_server_action(server['id'], {'os-start': None})
+        return self._wait_for_state_change(server, 'ACTIVE')
+
+    def _stop_server(self, server, wait_for_stop=True):
+        self.api.post_server_action(server['id'], {'os-stop': None})
+        if wait_for_stop:
+            return self._wait_for_state_change(server, 'SHUTOFF')
+        return server
+
+    def _snapshot_server(self, server, snapshot_name):
+        """Create server snapshot."""
+        self.api.post_server_action(
+            server['id'],
+            {'createImage': {'name': snapshot_name}}
+        )
+
+    def _attach_volumes(self, server, vol_ids: ty.List[str]):
+        # attach volumes to server
+        # these attachments are done by nova api, that means
+        # nova know about these attachments and so they are valid ones.
+
+        for vol_id in vol_ids:
+            self.api.post_server_volume(
+                server['id'], {'volumeAttachment': {'volumeId': vol_id}})
+
+        for vol_id in vol_ids:
+            # wait for each vol to get attached
+            self._wait_for_volume_attach(server['id'], vol_id)
+
+        # this is just to refresh server object
+        # so attached volumes can be shown in server
+        return self._show_server(server)
+
+    def _create_vol_attachments_by_cinder(
+            self, volume_id, server, new_attachments=1):
+        # these attachments are done by cinder api,
+        # and nova is not aware of them.
+
+        for _ in range(new_attachments):
+            self.cinder.create_vol_attachment(
+                volume_id, server['id'])
+
+    def _create_server_boot_from_volume(self, image_args=None,
+                                        flavor_id=None, networks=None):
+        bfv_image_id = uuids.bfv_image_uuid
+        timestamp = datetime.datetime(2011, 1, 1, 1, 2, 3)
+
+        image = {
+            'id': bfv_image_id,
+            'name': 'fake_image_name',
+            'created_at': timestamp,
+            'updated_at': timestamp,
+            'deleted_at': None,
+            'deleted': False,
+            'status': 'active',
+            'container_format': 'raw',
+            'disk_format': 'raw',
+            'min_disk': 0
+        }
+        if image_args:
+            image.update(image_args)
+
+        self.glance.create(None, image)
+
+        # for bfv, image is not required in server request
+        server = self._build_server(networks=networks, flavor_id=flavor_id)
+        server.pop('imageRef')
+
+        # as bfv-image will be used as source in block_device_mapping_v2
+        # here block device will be created based on bfv-image
+        # i.e bfvimage_id
+        server['block_device_mapping_v2'] = [{
+            'source_type': 'image',
+            'destination_type': 'volume',
+            'boot_index': 0,
+            'uuid': bfv_image_id,
+            'volume_size': 1,
+        }]
+        server = self.api.post_server({'server': server})
+        return self._wait_for_state_change(server, 'ACTIVE')
 
 
 class PlacementHelperMixin:
@@ -510,17 +752,25 @@ class PlacementHelperMixin:
         ).body['resource_classes']
         return [d['name'] for d in resp]
 
+    def _create_resource_class(self, resource_class):
+        return self.placement.post(
+            '/resource_classes', {"name": resource_class}, version='1.6')
+
     def _get_all_providers(self):
         return self.placement.get(
             '/resource_providers', version='1.14'
         ).body['resource_providers']
 
-    def _get_all_rp_uuids_in_a_tree(self, in_tree_rp_uuid):
+    def _get_all_rps_in_a_tree(self, in_tree_rp_uuid):
         rps = self.placement.get(
             '/resource_providers?in_tree=%s' % in_tree_rp_uuid,
             version='1.20',
         ).body['resource_providers']
-        return [rp['uuid'] for rp in rps]
+        return rps
+
+    def _get_all_rp_uuids_in_a_tree(self, in_tree_rp_uuid):
+        return [
+            rp['uuid'] for rp in self._get_all_rps_in_a_tree(in_tree_rp_uuid)]
 
     def _post_resource_provider(self, rp_name):
         return self.placement.post(
@@ -728,6 +978,20 @@ class PlacementHelperMixin:
             'Test expected a single migration but found %i' % len(migrations))
         return migrations[0].uuid
 
+    def _reserve_placement_resource(self, rp_name, rc_name, reserved):
+        rp_uuid = self._get_provider_uuid_by_name(rp_name)
+        inv = self.placement.get(
+            '/resource_providers/%s/inventories/%s' % (rp_uuid, rc_name),
+            version='1.26'
+        ).body
+        inv["reserved"] = reserved
+        result = self.placement.put(
+            '/resource_providers/%s/inventories/%s' % (rp_uuid, rc_name),
+            version='1.26', body=inv
+        ).body
+        self.assertEqual(reserved, result["reserved"])
+        return result
+
 
 class PlacementInstanceHelperMixin(InstanceHelperMixin, PlacementHelperMixin):
     """A placement-aware variant of InstanceHelperMixin."""
@@ -738,6 +1002,9 @@ class PlacementInstanceHelperMixin(InstanceHelperMixin, PlacementHelperMixin):
             '/allocations/%s' % server_uuid
         ).body['allocations']
 
+    def _delete_server_allocations(self, server_uuid):
+        self.placement.delete(f'/allocations/{server_uuid}')
+
     def _wait_for_server_allocations(self, consumer_id, max_retries=20):
         retry_count = 0
         while True:
@@ -864,7 +1131,7 @@ class PlacementInstanceHelperMixin(InstanceHelperMixin, PlacementHelperMixin):
         # instance.delete.end notification as that is emitted after the
         # resources are freed.
 
-        fake_notifier.wait_for_versioned_notifications('instance.delete.end')
+        self.notifier.wait_for_versioned_notifications('instance.delete.end')
 
         for rp_uuid in [
             self._get_provider_uuid_by_host(hostname)
@@ -1054,19 +1321,24 @@ class _IntegratedTestBase(test.TestCase, PlacementInstanceHelperMixin):
     #: do real authentication.
     STUB_KEYSTONE = True
 
+    #: Whether to treat RPC casts as calls. This shouldn't really default to
+    #: True but a significant number of existing tests may be relying on it.
+    CAST_AS_CALL = True
+
     def setUp(self):
         super(_IntegratedTestBase, self).setUp()
 
-        self.useFixture(cast_as_call.CastAsCall(self))
+        if self.CAST_AS_CALL:
+            self.useFixture(nova_fixtures.CastAsCallFixture(self))
 
         self.placement = self.useFixture(func_fixtures.PlacementFixture()).api
         self.neutron = self.useFixture(nova_fixtures.NeutronFixture(self))
         self.cinder = self.useFixture(nova_fixtures.CinderFixture(self))
         self.glance = self.useFixture(nova_fixtures.GlanceFixture(self))
-        self.policy = self.useFixture(policy_fixture.RealPolicyFixture())
+        self.policy = self.useFixture(nova_fixtures.RealPolicyFixture())
 
-        fake_notifier.stub_notifier(self)
-        self.addCleanup(fake_notifier.reset)
+        self.notifier = self.useFixture(
+            nova_fixtures.NotificationFixture(self))
 
         self._setup_services()
 
@@ -1079,6 +1351,10 @@ class _IntegratedTestBase(test.TestCase, PlacementInstanceHelperMixin):
     def _setup_conductor_service(self):
         return self.start_service('conductor')
 
+    def _stop_computes(self):
+        for compute in self.computes.values():
+            compute.stop()
+
     def _setup_services(self):
         # NOTE(danms): Set the global MQ connection to that of our first cell
         # for any cells-ignorant code. Normally this is defaulted in the tests
@@ -1087,8 +1363,11 @@ class _IntegratedTestBase(test.TestCase, PlacementInstanceHelperMixin):
             self.flags(transport_url=self.cell_mappings['cell1'].transport_url)
 
         self.conductor = self._setup_conductor_service()
+        self.addCleanup(self.conductor.stop)
         self.scheduler = self._setup_scheduler_service()
+        self.addCleanup(self.scheduler.stop)
         self.compute = self._setup_compute_service()
+        self.addCleanup(self._stop_computes)
 
         self.api_fixture = self.useFixture(
             nova_fixtures.OSAPIFixture(
@@ -1096,13 +1375,13 @@ class _IntegratedTestBase(test.TestCase, PlacementInstanceHelperMixin):
                 use_project_id_in_urls=self.USE_PROJECT_ID,
                 stub_keystone=self.STUB_KEYSTONE))
 
+        self.admin_api = self.api_fixture.admin_api
         # if the class needs to run as admin, make the api endpoint
         # the admin, otherwise it's safer to run as non admin user.
         if self.ADMIN_API:
             self.api = self.api_fixture.admin_api
         else:
             self.api = self.api_fixture.api
-            self.admin_api = self.api_fixture.admin_api
 
         if self.microversion:
             self.api.microversion = self.microversion
@@ -1130,14 +1409,15 @@ class ProviderUsageBaseTestCase(test.TestCase, PlacementInstanceHelperMixin):
         self.flags(compute_driver=self.compute_driver)
         super(ProviderUsageBaseTestCase, self).setUp()
 
-        self.policy = self.useFixture(policy_fixture.RealPolicyFixture())
+        self.policy = self.useFixture(nova_fixtures.RealPolicyFixture())
         self.neutron = self.useFixture(nova_fixtures.NeutronFixture(self))
         self.glance = self.useFixture(nova_fixtures.GlanceFixture(self))
+        self.useFixture(nova_fixtures.CinderFixture(self))
         self.placement = self.useFixture(func_fixtures.PlacementFixture()).api
         self.useFixture(nova_fixtures.AllServicesCurrent())
 
-        fake_notifier.stub_notifier(self)
-        self.addCleanup(fake_notifier.reset)
+        self.notifier = self.useFixture(
+            nova_fixtures.NotificationFixture(self))
 
         self.api_fixture = self.useFixture(nova_fixtures.OSAPIFixture(
             api_version='v2.1'))
