@@ -13,9 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import ddt
 import fixtures
-import mock
 from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import uuidutils
 import webob
@@ -39,16 +40,6 @@ CONF = nova.conf.CONF
 FAKE_UUID = fakes.FAKE_UUID
 
 
-class MockSetAdminPassword(object):
-    def __init__(self):
-        self.instance_id = None
-        self.password = None
-
-    def __call__(self, context, instance, password):
-        self.instance_id = instance['uuid']
-        self.password = password
-
-
 @ddt.ddt
 class ServerActionsControllerTestV21(test.TestCase):
     image_uuid = '76fa36fc-c930-4bf3-8c8a-ea2a2420deb6'
@@ -76,11 +67,11 @@ class ServerActionsControllerTestV21(test.TestCase):
 
         self.controller = self._get_controller()
         self.compute_api = self.controller.compute_api
-        # We don't care about anything getting as far as hitting the compute
-        # RPC API so we just mock it out here.
-        mock_rpcapi = mock.patch.object(self.compute_api, 'compute_rpcapi')
-        mock_rpcapi.start()
-        self.addCleanup(mock_rpcapi.stop)
+        # In most of the cases we don't care about anything getting as far as
+        # hitting the compute RPC API so we just mock it out here.
+        patcher_rpcapi = mock.patch.object(self.compute_api, 'compute_rpcapi')
+        self.mock_rpcapi = patcher_rpcapi.start()
+        self.addCleanup(patcher_rpcapi.stop)
         # The project_id here matches what is used by default in
         # fake_compute_get which need to match for policy checks.
         self.req = fakes.HTTPRequest.blank('',
@@ -100,10 +91,12 @@ class ServerActionsControllerTestV21(test.TestCase):
             self.controller.compute_api, 'compute_task_api')
         mock_conductor.start()
         self.addCleanup(mock_conductor.stop)
-        # Assume that none of the tests are using ports with resource requests.
-        self.mock_list_port = self.useFixture(
-            fixtures.MockPatch('nova.network.neutron.API.list_ports')).mock
-        self.mock_list_port.return_value = {'ports': []}
+        self.mock_neutron_extension_list = self.useFixture(
+            fixtures.MockPatch(
+                'nova.network.neutron.API._refresh_neutron_extensions_cache'
+            )
+        ).mock
+        self.mock_neutron_extension_list.return_value = {'extensions': []}
 
     def _get_controller(self):
         return self.servers.ServersController()
@@ -141,7 +134,13 @@ class ServerActionsControllerTestV21(test.TestCase):
                               body=body_map.get(action))
             expected_attrs = ['flavor', 'numa_topology']
             if method == 'resize':
-                expected_attrs.append('services')
+                expected_attrs.extend(['services', 'resources',
+                                       'pci_requests', 'pci_devices',
+                                       'trusted_certs', 'vcpu_model'])
+            elif method == 'rebuild':
+                expected_attrs.extend(['trusted_certs', 'pci_requests',
+                                       'pci_devices', 'resources',
+                                       'migration_context'])
             mock_get.assert_called_once_with(self.context, uuid,
                 expected_attrs=expected_attrs,
                 cell_down_support=False)
@@ -163,7 +162,9 @@ class ServerActionsControllerTestV21(test.TestCase):
                     '_action_reboot': {'reboot': {'type': 'HARD'}},
                     '_action_rebuild': {'rebuild': {
                                 'imageRef': self.image_uuid,
-                                'adminPass': 'TNc53Dr8s7vw'}}}
+                                'adminPass': 'TNc53Dr8s7vw'}},
+                    '_action_revert_resize': {'revertResize': None},
+                    '_action_confirm_resize': {'confirmResize': None}}
 
         args_map = {'_action_resize': (('2'), {'auto_disk_config': None}),
                     '_action_confirm_resize': ((), {}),
@@ -267,7 +268,7 @@ class ServerActionsControllerTestV21(test.TestCase):
     def _test_rebuild_preserve_ephemeral(self, value=None):
         return_server = fakes.fake_compute_get(
                 project_id=fakes.FAKE_PROJECT_ID,
-                image_ref='2',
+                image_ref=uuids.image_ref,
                 vm_state=vm_states.ACTIVE,
                 host='fake_host')
         self.stub_out('nova.compute.api.API.get', return_server)
@@ -302,7 +303,7 @@ class ServerActionsControllerTestV21(test.TestCase):
     def test_rebuild_accepted_minimum(self):
         return_server = fakes.fake_compute_get(
                 project_id=fakes.FAKE_PROJECT_ID,
-                image_ref='2',
+                image_ref=uuids.image_ref,
                 vm_state=vm_states.ACTIVE, host='fake_host')
         self.stub_out('nova.compute.api.API.get', return_server)
         self_href = 'http://localhost/v2/servers/%s' % FAKE_UUID
@@ -316,7 +317,7 @@ class ServerActionsControllerTestV21(test.TestCase):
         robj = self.controller._action_rebuild(self.req, FAKE_UUID, body=body)
         body = robj.obj
 
-        self.assertEqual(body['server']['image']['id'], '2')
+        self.assertEqual(body['server']['image']['id'], uuids.image_ref)
         self.assertEqual(len(body['server']['adminPass']),
                          CONF.password_length)
 
@@ -361,7 +362,7 @@ class ServerActionsControllerTestV21(test.TestCase):
 
         return_server = fakes.fake_compute_get(
                 project_id=fakes.FAKE_PROJECT_ID,
-                image_ref='2',
+                image_ref=uuids.image_ref,
                 vm_state=vm_states.ACTIVE, host='fake_host')
         self.stub_out('nova.compute.api.API.get', return_server)
         self_href = 'http://localhost/v2/servers/%s' % FAKE_UUID
@@ -375,7 +376,7 @@ class ServerActionsControllerTestV21(test.TestCase):
         robj = self.controller._action_rebuild(self.req, FAKE_UUID, body=body)
         body = robj.obj
 
-        self.assertEqual(body['server']['image']['id'], '2')
+        self.assertEqual(body['server']['image']['id'], uuids.image_ref)
         self.assertNotIn("adminPass", body['server'])
 
         self.assertEqual(robj['location'], self_href)
@@ -384,8 +385,6 @@ class ServerActionsControllerTestV21(test.TestCase):
 
     @ddt.data(
         exception.InstanceIsLocked(instance_uuid=uuids.instance),
-        exception.OperationNotSupportedForVTPM(
-            instance_uuid=uuids.instance, operation='foo'),
     )
     @mock.patch('nova.compute.api.API.rebuild')
     def test_rebuild__http_conflict_error(self, exc, mock_rebuild):
@@ -396,6 +395,22 @@ class ServerActionsControllerTestV21(test.TestCase):
             self.req, uuids.instance,
             body={'rebuild': {'imageRef': uuids.image}})
 
+    @ddt.data(
+        exception.ForbiddenWithAccelerators(),
+        exception.OperationNotSupportedForVTPM(
+            instance_uuid=uuids.instance, operation='foo'),
+        exception.OperationNotSupportedForVDPAInterface(
+            instance_uuid=uuids.instance, operation='foo'),
+    )
+    @mock.patch('nova.compute.api.API.rebuild')
+    def test_rebuild_raises_badrequest_for_not_supported_features(
+        self, exc, mock_rebuild):
+        mock_rebuild.side_effect = exc
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller._action_rebuild,
+                          self.req, uuids.instance,
+                          body={'rebuild': {'imageRef': uuids.image}})
+
     def test_rebuild_raises_conflict_on_invalid_state(self):
         body = {'rebuild': {'imageRef': uuids.image}}
 
@@ -471,7 +486,7 @@ class ServerActionsControllerTestV21(test.TestCase):
     def test_rebuild_admin_pass(self):
         return_server = fakes.fake_compute_get(
                 project_id=fakes.FAKE_PROJECT_ID,
-                image_ref='2',
+                image_ref=uuids.image_ref,
                 vm_state=vm_states.ACTIVE, host='fake_host')
         self.stub_out('nova.compute.api.API.get', return_server)
 
@@ -485,7 +500,7 @@ class ServerActionsControllerTestV21(test.TestCase):
         body = self.controller._action_rebuild(self.req, FAKE_UUID,
                                                body=body).obj
 
-        self.assertEqual(body['server']['image']['id'], '2')
+        self.assertEqual(body['server']['image']['id'], uuids.image_ref)
         self.assertEqual(body['server']['adminPass'], 'asdf')
 
     def test_rebuild_admin_pass_pass_disabled(self):
@@ -495,7 +510,7 @@ class ServerActionsControllerTestV21(test.TestCase):
 
         return_server = fakes.fake_compute_get(
                 project_id=fakes.FAKE_PROJECT_ID,
-                image_ref='2',
+                image_ref=FAKE_UUID,
                 vm_state=vm_states.ACTIVE, host='fake_host')
         self.stub_out('nova.compute.api.API.get', return_server)
 
@@ -509,7 +524,7 @@ class ServerActionsControllerTestV21(test.TestCase):
         body = self.controller._action_rebuild(self.req, FAKE_UUID,
                                                body=body).obj
 
-        self.assertEqual(body['server']['image']['id'], '2')
+        self.assertEqual(body['server']['image']['id'], FAKE_UUID)
         self.assertNotIn('adminPass', body['server'])
 
     def test_rebuild_server_not_found(self):
@@ -576,12 +591,20 @@ class ServerActionsControllerTestV21(test.TestCase):
 
         def return_image_meta(*args, **kwargs):
             image_meta_table = {
-                '2': {'id': uuids.image_id, 'status': 'active',
-                      'container_format': 'ari'},
-                '155d900f-4e14-4e4c-a73d-069cbf4541e6':
-                     {'id': uuids.image_id, 'status': 'active',
-                      'container_format': 'raw',
-                      'properties': {'kernel_id': 1, 'ramdisk_id': 2}},
+                uuids.image_1_id: {
+                    'id': uuids.image_1_id,
+                    'status': 'active',
+                    'container_format': 'ari'
+                },
+                uuids.image_2_id: {
+                    'id': uuids.image_2_id,
+                    'status': 'active',
+                    'container_format': 'raw',
+                    'properties': {
+                        'kernel_id': uuids.kernel_id,
+                        'ramdisk_id': uuids.ramdisk_id
+                    }
+                },
             }
             image_id = args[2]
             try:
@@ -595,7 +618,7 @@ class ServerActionsControllerTestV21(test.TestCase):
                       return_image_meta)
         body = {
             "rebuild": {
-                "imageRef": "155d900f-4e14-4e4c-a73d-069cbf4541e6",
+                "imageRef": uuids.image_2_id,
             },
         }
         self.assertRaises(webob.exc.HTTPBadRequest,
@@ -856,6 +879,23 @@ class ServerActionsControllerTestV21(test.TestCase):
                           self.controller._action_resize,
                           self.req, FAKE_UUID, body=body)
 
+    @mock.patch('nova.compute.api.API.resize',
+                side_effect=exception.ForbiddenWithAccelerators)
+    def test_resize_raises_badrequest_for_accelerator(self, mock_resize):
+        body = dict(resize=dict(flavorRef="http://localhost/3"))
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller._action_resize,
+                          self.req, FAKE_UUID, body=body)
+
+    @mock.patch('nova.compute.api.API.resize',
+                side_effect=exception.OperationNotSupportedForVDPAInterface(
+                    instance_uuid=FAKE_UUID, operation='foo'))
+    def test_resize_raises_badrequest_for_vdpaInterface(self, mock_resize):
+        body = dict(resize=dict(flavorRef="http://localhost/3"))
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller._action_resize,
+                          self.req, FAKE_UUID, body=body)
+
     def test_confirm_resize_server(self):
         body = dict(confirmResize=None)
 
@@ -1025,8 +1065,9 @@ class ServerActionsControllerTestV21(test.TestCase):
                          'delete_on_termination': False,
                          'no_device': None})]
 
-        self.stub_out('nova.db.api.block_device_mapping_get_all_by_instance',
-                      fake_block_device_mapping_get_all_by_instance)
+        self.stub_out(
+            'nova.db.main.api.block_device_mapping_get_all_by_instance',
+            fake_block_device_mapping_get_all_by_instance)
 
         system_metadata = dict(image_kernel_id=_fake_id('b'),
                                image_ramdisk_id=_fake_id('c'),
@@ -1047,21 +1088,23 @@ class ServerActionsControllerTestV21(test.TestCase):
 
         snapshot = dict(id=_fake_id('d'))
 
+        self.mock_rpcapi.quiesce_instance.side_effect = (
+            exception.InstanceQuiesceNotSupported(
+                instance_id="fake", reason="test"
+            )
+        )
+
         with test.nested(
             mock.patch.object(
                 self.controller.compute_api.volume_api, 'get_absolute_limits',
                 return_value={'totalSnapshotsUsed': 0,
                               'maxTotalSnapshots': 10}),
-            mock.patch.object(self.controller.compute_api.compute_rpcapi,
-                'quiesce_instance',
-                side_effect=exception.InstanceQuiesceNotSupported(
-                    instance_id='fake', reason='test')),
             mock.patch.object(self.controller.compute_api.volume_api, 'get',
                               return_value=volume),
             mock.patch.object(self.controller.compute_api.volume_api,
                               'create_snapshot_force',
                               return_value=snapshot),
-        ) as (mock_get_limits, mock_quiesce, mock_vol_get, mock_vol_create):
+        ) as (mock_get_limits, mock_vol_get, mock_vol_create):
 
             if mock_vol_create_side_effect:
                 mock_vol_create.side_effect = mock_vol_create_side_effect
@@ -1093,7 +1136,7 @@ class ServerActionsControllerTestV21(test.TestCase):
             for k in extra_properties.keys():
                 self.assertEqual(properties[k], extra_properties[k])
 
-            mock_quiesce.assert_called_once_with(mock.ANY, mock.ANY)
+            self.mock_rpcapi.quiesce_instance.assert_called_once()
             mock_vol_get.assert_called_once_with(mock.ANY, volume['id'])
             mock_vol_create.assert_called_once_with(mock.ANY, volume['id'],
                                                     mock.ANY, mock.ANY)
@@ -1137,8 +1180,9 @@ class ServerActionsControllerTestV21(test.TestCase):
                          'delete_on_termination': False,
                          'no_device': None})]
 
-        self.stub_out('nova.db.api.block_device_mapping_get_all_by_instance',
-                      fake_block_device_mapping_get_all_by_instance)
+        self.stub_out(
+            'nova.db.main.api.block_device_mapping_get_all_by_instance',
+            fake_block_device_mapping_get_all_by_instance)
 
         instance = fakes.fake_compute_get(
             project_id=fakes.FAKE_PROJECT_ID,
@@ -1156,21 +1200,23 @@ class ServerActionsControllerTestV21(test.TestCase):
 
         snapshot = dict(id=_fake_id('d'))
 
+        self.mock_rpcapi.quiesce_instance.side_effect = (
+            exception.InstanceQuiesceNotSupported(
+                instance_id="fake", reason="test"
+            )
+        )
+
         with test.nested(
             mock.patch.object(
                 self.controller.compute_api.volume_api, 'get_absolute_limits',
                 return_value={'totalSnapshotsUsed': 0,
                               'maxTotalSnapshots': 10}),
-            mock.patch.object(self.controller.compute_api.compute_rpcapi,
-                'quiesce_instance',
-                side_effect=exception.InstanceQuiesceNotSupported(
-                    instance_id='fake', reason='test')),
             mock.patch.object(self.controller.compute_api.volume_api, 'get',
                               return_value=volume),
             mock.patch.object(self.controller.compute_api.volume_api,
                               'create_snapshot_force',
                               return_value=snapshot),
-        ) as (mock_get_limits, mock_quiesce, mock_vol_get, mock_vol_create):
+        ) as (mock_get_limits, mock_vol_get, mock_vol_create):
 
             response = self.controller._action_create_image(self.req,
                 FAKE_UUID, body=body)
@@ -1185,7 +1231,7 @@ class ServerActionsControllerTestV21(test.TestCase):
                 for key, val in extra_metadata.items():
                     self.assertEqual(properties[key], val)
 
-            mock_quiesce.assert_called_once_with(mock.ANY, mock.ANY)
+            self.mock_rpcapi.quiesce_instance.assert_called_once()
             mock_vol_get.assert_called_once_with(mock.ANY, volume['id'])
             mock_vol_create.assert_called_once_with(mock.ANY, volume['id'],
                                                     mock.ANY, mock.ANY)
@@ -1274,21 +1320,3 @@ class ServerActionsControllerTestV21(test.TestCase):
         self.assertRaises(webob.exc.HTTPConflict,
                           self.controller._action_create_image,
                           self.req, FAKE_UUID, body=body)
-
-    @mock.patch('nova.objects.Service.get_by_host_and_binary')
-    @mock.patch('nova.api.openstack.common.'
-                'instance_has_port_with_resource_request', return_value=True)
-    def test_resize_with_bandwidth_from_old_compute_not_supported(
-            self, mock_has_res_req, mock_get_service):
-        body = dict(resize=dict(flavorRef="http://localhost/3"))
-        mock_get_service.return_value = objects.Service()
-        mock_get_service.return_value.version = 38
-
-        self.assertRaises(webob.exc.HTTPConflict,
-                          self.controller._action_resize,
-                          self.req, FAKE_UUID, body=body)
-
-        mock_has_res_req.assert_called_once_with(
-            FAKE_UUID, self.controller.network_api)
-        mock_get_service.assert_called_once_with(
-            self.req.environ['nova.context'], 'fake_host', 'nova-compute')
