@@ -871,6 +871,67 @@ class HostTestCase(test.NoDBTestCase):
         self.assertEqual(unversioned_caps, versioned_caps)
         self.assertIs(unversioned_caps, versioned_caps)
 
+    def test_get_domain_capabilities_without_vtpm(self):
+        caps = self._test_get_domain_capabilities()
+        self.assertEqual(vconfig.LibvirtConfigDomainCaps, type(caps))
+        self.assertIsNone(caps.devices.tpm)
+
+    @mock.patch.object(
+        fakelibvirt.virConnect, '_domain_capability_devices', new=
+        fakelibvirt.virConnect._domain_capability_devices_with_tpm_supported
+    )
+    def test_get_domain_capabilities_with_vtpm_supported(self):
+        caps = self._test_get_domain_capabilities()
+        self.assertEqual(vconfig.LibvirtConfigDomainCaps, type(caps))
+        self.assertEqual(vconfig.LibvirtConfigDomainCapsTpm,
+                         type(caps.devices.tpm))
+        self.assertTrue(caps.devices.tpm.supported)
+        self.assertEqual(
+            ['tpm-tis', 'tpm-crb'],
+            caps.devices.tpm.models
+        )
+        self.assertEqual(
+            ['passthrough', 'emulator', 'external'],
+            caps.devices.tpm.backend_models
+        )
+        self.assertIsNone(caps.devices.tpm.backend_versions)
+
+    @mock.patch.object(
+        fakelibvirt.virConnect, '_domain_capability_devices', new=
+        fakelibvirt.virConnect._domain_capability_devices_with_tpm_unsupported
+    )
+    def test_get_domain_capabilities_with_vtpm_unsupported(self):
+        caps = self._test_get_domain_capabilities()
+        self.assertEqual(vconfig.LibvirtConfigDomainCaps, type(caps))
+        self.assertEqual(vconfig.LibvirtConfigDomainCapsTpm,
+                         type(caps.devices.tpm))
+        self.assertFalse(caps.devices.tpm.supported)
+        self.assertEqual([], caps.devices.tpm.backend_models)
+        self.assertIsNone(caps.devices.tpm.backend_versions)
+
+    @mock.patch.object(
+        fakelibvirt.virConnect, '_domain_capability_devices', new=
+        fakelibvirt.virConnect._domain_capability_devices_with_tpm_versions
+    )
+    def test_get_domain_capabilities_with_vtpm_supported_and_versions(self):
+        caps = self._test_get_domain_capabilities()
+        self.assertEqual(vconfig.LibvirtConfigDomainCaps, type(caps))
+        self.assertEqual(vconfig.LibvirtConfigDomainCapsTpm,
+                         type(caps.devices.tpm))
+        self.assertTrue(caps.devices.tpm.supported)
+        self.assertEqual(
+            ['tpm-tis'],
+            caps.devices.tpm.models
+        )
+        self.assertEqual(
+            ['passthrough', 'emulator', 'external'],
+            caps.devices.tpm.backend_models
+        )
+        self.assertEqual(
+            ['2.0'],
+            caps.devices.tpm.backend_versions
+        )
+
     @mock.patch.object(fakelibvirt.virConnect, '_domain_capability_features',
                        new='')
     def test_get_domain_capabilities_no_features(self):
@@ -890,6 +951,8 @@ class HostTestCase(test.NoDBTestCase):
         if supported:
             self.assertEqual(47, sev.cbitpos)
             self.assertEqual(1, sev.reduced_phys_bits)
+            self.assertIsNone(sev.max_guests)
+            self.assertIsNone(sev.max_es_guests)
 
     @mock.patch.object(
         fakelibvirt.virConnect, '_domain_capability_features', new=
@@ -904,6 +967,22 @@ class HostTestCase(test.NoDBTestCase):
     def test_get_domain_capabilities_sev_supported(self):
         self._test_get_domain_capabilities_sev(True)
 
+    @mock.patch.object(
+        fakelibvirt.virConnect, '_domain_capability_features', new=
+        fakelibvirt.virConnect._domain_capability_features_with_SEV_max_guests)
+    def test_get_domain_capabilities_sev_max_guests(self):
+        caps = self._test_get_domain_capabilities()
+        self.assertEqual(vconfig.LibvirtConfigDomainCaps, type(caps))
+        features = caps.features
+        self.assertEqual(1, len(features))
+        sev = features[0]
+        self.assertEqual(vconfig.LibvirtConfigDomainCapsFeatureSev, type(sev))
+        self.assertTrue(sev.supported)
+        self.assertEqual(47, sev.cbitpos)
+        self.assertEqual(1, sev.reduced_phys_bits)
+        self.assertEqual(100, sev.max_guests)
+        self.assertEqual(15, sev.max_es_guests)
+
     @mock.patch.object(fakelibvirt.virConnect, "getHostname")
     def test_get_hostname_caching(self, mock_hostname):
         mock_hostname.return_value = "foo"
@@ -1150,6 +1229,18 @@ Active:          8381604 kB
         self.host.device_lookup_by_name("foo")
         mock_nodeDeviceLookupByName.assert_called_once_with("foo")
 
+    @mock.patch.object(fakelibvirt.virConnect, 'nodeDeviceCreateXML')
+    def test_device_create(self, mock_create):
+        node_dev = vconfig.LibvirtConfigNodeDevice()
+        self.host.device_create(node_dev)
+        mock_create.assert_called_once_with(node_dev.to_xml(), flags=0)
+
+    @mock.patch.object(fakelibvirt.virConnect, 'nodeDeviceDefineXML')
+    def test_device_define(self, mock_define):
+        node_dev = vconfig.LibvirtConfigNodeDevice()
+        self.host.device_define(node_dev)
+        mock_define.assert_called_once_with(node_dev.to_xml(), flags=0)
+
     def test_get_pcinet_info(self):
         dev_name = "net_enp2s2_02_9a_a1_37_be_54"
         parent_address = "pci_0000_04_11_7"
@@ -2154,48 +2245,6 @@ class LibvirtTpoolProxyTestCase(test.NoDBTestCase):
             self.assertIsInstance(domain, tpool.Proxy)
             self.assertIn(domain.UUIDString(), (uuids.vm1, uuids.vm2))
 
-    def _add_fake_host_devices(self):
-        self.conn._obj.pci_info = fakelibvirt.HostPCIDevicesInfo(
-            num_pci=1, num_pfs=2, num_vfs=2, num_mdevcap=3)
-        mdevs = {
-            'mdev_4b20d080_1b54_4048_85b3_a6a62d165c01':
-                fakelibvirt.FakeMdevDevice(
-                    dev_name='mdev_4b20d080_1b54_4048_85b3_a6a62d165c01',
-                    type_id=fakelibvirt.NVIDIA_11_VGPU_TYPE,
-                    parent=fakelibvirt.MDEVCAP_DEV1_PCI_ADDR),
-        }
-        self.conn._obj.mdev_info = fakelibvirt.HostMdevDevicesInfo(
-            devices=mdevs)
-
-    def test_tpool_list_all_devices(self):
-        self._add_fake_host_devices()
-        devs = self.host.list_all_devices(
-            fakelibvirt.VIR_CONNECT_LIST_NODE_DEVICES_CAP_PCI_DEV)
-        self.assertEqual(8, len(devs))
-        for dev in devs:
-            self.assertIsInstance(dev, tpool.Proxy)
-
-    def test_tpool_list_pci_devices(self):
-        self._add_fake_host_devices()
-        dev_names = self.host.list_pci_devices()
-        self.assertEqual(8, len(dev_names))
-        for name in dev_names:
-            self.assertIsInstance(name, str)
-
-    def test_tpool_list_mdev_capable_devices(self):
-        self._add_fake_host_devices()
-        dev_names = self.host.list_mdev_capable_devices()
-        self.assertEqual(3, len(dev_names))
-        for name in dev_names:
-            self.assertIsInstance(name, str)
-
-    def test_tpool_list_mediated_devices(self):
-        self._add_fake_host_devices()
-        dev_names = self.host.list_mediated_devices()
-        self.assertEqual(1, len(dev_names))
-        for name in dev_names:
-            self.assertIsInstance(name, str)
-
 
 class LoadersTestCase(test.NoDBTestCase):
 
